@@ -273,9 +273,8 @@ as a templating language but not an application server.
 
 C<handle_component()> will create a CGI query object, parse the query
 parameters, and send the HTTP header and component output to STDOUT.
-If you want to handle those parts yourself, see L<Using
-Mason from a standalone script|HTML::Mason::Interp/"Using Mason
-from a standalone script">.
+If you want to handle those parts yourself, see
+the L<Using Mason from a standalone script|HTML::Mason::Admin/Using Mason from a standalone script> section of the administrator's manual.
 
 =item * handle_cgi_object()
 
