@@ -102,12 +102,6 @@ sub compiled_component
 	$subs{main} = $params->{code};
 	$params->{code} = "sub {\n\$m->call_dynamic( 'main', \@_ )\n}";
 
-        if ( exists $params->{filter} )
-        {
-            $subs{filter} = $params->{filter};
-            $params->{filter} = "sub {\n\$m->call_dynamic( 'filter', \@_ )\n}";
-        }
-
 	$params->{dynamic_subs_init} =
 	    join '', ( "sub {\n",
 		       $self->_blocks('shared'),
@@ -194,6 +188,7 @@ sub _make_main_header
     my $self = shift;
 
     my $pkg = $self->in_package;
+
     return join '', ( "package $pkg;\n",
 		      $self->use_strict ? "use strict;\n" : "no strict;\n",
 		      sprintf( "use vars qw(\%s);\n",
@@ -259,18 +254,7 @@ sub _component_params
     $params{declared_args} = join '', "{\n", $self->_declared_args, "\n}"
 	if @{ $self->{current_comp}{args} };
 
-    if ( my @filter = $self->_blocks('filter') )
-    {
-        $params{filter} =
-            ( join '',
-              "sub { local \$_ = shift;\n",
-              "my %ARGS; { local \$^W; %ARGS = \@_ unless \@_ % 2; }\n",
-              ( join ";\n", @filter ),
-              ";\n",
-              "return \$_;\n",
-              "}\n",
-            );
-    }
+    $params{has_filter} = 1 if $self->_blocks('filter');
 
     return \%params;
 }
@@ -296,6 +280,7 @@ EOF
     return join '', ( $self->preamble,
 		      "my \%ARGS;\n",
 		      @args,
+                      $self->_filter,
 		      "\$m->debug_hook( \$m->current_comp->path ) if ( \%DB:: );\n\n",
 		      $self->_blocks('init'),
 		      $self->{current_comp}{body},
@@ -374,6 +359,24 @@ EOF
     return @req_check, $decl, @assign;
 }
 
+sub _filter
+{
+    my $self = shift;
+
+    my @filter;
+    @filter = $self->_blocks('filter')
+        or return;
+
+    return ( join '',
+             "\$m->current_comp->filter( sub { local \$_ = shift;\n",
+             ( join ";\n", @filter ),
+             ";\n",
+             "return \$_;\n",
+             "} );\n",
+           );
+
+}
+
 sub _flags
 {
     my $self = shift;
