@@ -1,4 +1,4 @@
-# Copyright (c) 1998-2005 by Jonathan Swartz. All rights reserved.
+# Copyright (c) 1998-2002 by Jonathan Swartz. All rights reserved.
 # This program is free software; you can redistribute it and/or modify it
 # under the same terms as Perl itself.
 
@@ -50,11 +50,8 @@ HTML::Mason::Escapes - Functions to escape text for Mason
 
 =head1 DESCRIPTION
 
-This module contains functions for implementing Mason's L<substitution
-escaping|HTML::Mason::Devel/Escaping expressions> feature.  These
-functions may also be called directly.
-
-=over 4
+This module contains exportable functions that are intended to be used
+by other Mason modules.
 
 =item html_entities_escape
 
@@ -67,10 +64,10 @@ the section on L<User-defined Escapes in the Developer's
 Manual|HTML::Mason::Devel/User-defined Escapes> for more details on
 how to do this.
 
-=item url_escape
+=item uri_escape
 
 This takes a scalar reference and replaces any text it contains
-matching C<[^a-zA-Z0-9_.-]> with the URL-escaped equivalent, a percent
+matching C<[^a-zA-Z0-9_.-]> with the URI-escaped equivalent, a percent
 sign (%) followed by the hexadecimal number of that character.
 
 =item basic_html_escape
@@ -80,7 +77,7 @@ the following characters: '&', '>', '<', and '"'.
 
 It is provided for those who wish to use it to replace (or supplement)
 the existing 'h' escape flag, via the Interpreter's L<C<set_escape()>
-method|HTML::Mason::Interp/item_set_escape>.
+method|HTML>::Mason::Interp/item_set_escape>.
 
 This function is provided in order to allow people to return the HTML
 escaping behavior in 1.0x.  However, this behavior presents a
@@ -94,6 +91,6 @@ scripting attacks.  See
 http://www.megasecurity.org/Info/cross-site_scripting.txt for more
 details.
 
-=back
+It is not exportable.
 
 =cut
