@@ -172,7 +172,7 @@ order whenever a component path must be resolved to a filesystem path.
 =item comp_root
 
 This method returns the component root, which will either be a scalar
-or an array reference, as documented in L<CONSTRUCTOR|"CONSTRUCTOR">.
+or an array reference, as documented L<here|HTML::Mason::Resolver::File/item_comp_root>.
 
 =back
 
